package org.supermmx.ginger;

import org.supermmx.ginger.gui.GingerFrame;

import org.supermmx.peony.ui.PeonyUIData;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory; 

import org.xnap.commons.i18n.I18nFactory;
import org.xnap.commons.i18n.I18n;

import java.util.List;
import java.util.ArrayList;
import java.util.Locale;

/**
 * Ginger main class.
 *
 * @author SuperMMX
 */
public class Ginger implements PeonyUIData {
    /**
     * Ginger resource domain
     */
    public static final String GINGER_RESOURCE = "ginger";

    private static Log log = LogFactory.getLog(Ginger.class);

    /**
     * Current locale
     */
    private Locale locale = Locale.getDefault();

    private GingerFrame frame;

    private GingerSession emptySession;

    public Ginger() {
        emptySession = new GingerSession();
        frame = new GingerFrame(this);

        // default empty session
        frame.addSession(emptySession);
    }

    /**
     * Version
     */
    public String getVersion() {
        return Version.VERSION;
    }

    /**
     * Revision number
     */
    public String getRevisionNumber() {
        return Version.REVISION_NUMBER;
    }

    /**
     * Revision ID
     */
    public String getRevisionID() {
        return Version.REVISION_ID;
    }

    /**
     * Main Frame
     */
    public GingerFrame getFrame() {
        return frame;
    }

    /**
     * Return the empty session.
     */
    public GingerSession getEmptySession() {
        return emptySession;
    }

    /**
     * Exit ginger.
     */
    public void exit() {
        // disconnect all sessions
        for (GingerSession session : frame.getSessions()) {
            session.disconnect();
        }

        // dispose the frame
        frame.dispose();

        // JVM shoule exit automatically
    }

    /**
     * ================
     *   PeonyUI Data
     * ================
     */

    /**
     * Get the I18n instance for the specified class.
     *
     * @param clazz the class
     *
     * @return the I18n instance for the class in current locale
     */
    public I18n getI18n(Class clazz) {
        return I18nFactory.getI18n(clazz, GINGER_RESOURCE,
                                   locale, I18nFactory.FALLBACK);
    }

    /**
     * Ginger itself as the data.
     */
    public Object getData() {
        return this;
    }

    public static void main(String[] args) {
        Ginger ginger = new Ginger();
    }
}
