package org.supermmx.ginger;

import org.supermmx.ginger.document.GingerDocument;
import org.supermmx.ginger.view.GingerView;

import org.supermmx.peony.net.PeonyChannel;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory; 

/**
 * A connection session.
 *
 * @author SuperMMX
 */
public class GingerSession {
    private static Log log = LogFactory.getLog(GingerSession.class);

    /**
     * Session name
     */
    private String name;

    private GingerDestination dest;
    private GingerTelnet telnet;
    private GingerDocument document;
    private GingerView view;

    private PeonyChannel channel;

    public GingerSession() {
        telnet = new GingerTelnet(this);
        document = new GingerDocument(this);
        view = new GingerView(this);
    }

    public GingerSession(GingerDestination dest) {
        this();

        setDestination(dest);
    }

    /**
     * Get the session name
     *
     * @return the session name
     */
    public String getName() {
        if (name != null) {
            return name;
        }

        if (dest != null) {
            return dest.getHost();
        }

        return null;
    }

    /**
     * Set the session name.
     *
     * @param name session name
     */
    public void setName(String name) {
        this.name = name;
    }

    public void connect() {
        if (channel == null) {
            // throw exception ?
            log.error("Destination not set");

            return ;
        }

        try {
            log.debug("Connect");

            channel.setHost(dest.getHost());
            channel.setPort(dest.getPort());

            channel.connect();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void disconnect() {
        if (channel == null) {
            return ;
        }

        try {
            log.debug("Disconnect");
            channel.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public boolean isConnected() {
        if (channel == null) {
            return false;
        }

        return channel.isConnected();
    }

    /**
     * Get the underlying document.
     *
     * @return the document
     */
    public GingerDocument getDocument() {
        return document;
    }

    /**
     * Get the document view.
     *
     * @return the document view
     */
    public GingerView getView() {
        return view;
    }

    /**
     * Get destination
     */
    public GingerDestination getDestination() {
        return dest;
    }

    /**
     * Set destination. Will reset the underlying connection.
     */
    public void setDestination(GingerDestination dest) {
        this.dest = dest;

        if (channel == null) {
            channel = new PeonyChannel(dest.getHost(), dest.getPort());
            channel.setChannelCallback(telnet);
        }
    }

    public void sendRawData(byte[] buf) {
        try {
            if (channel != null && channel.isConnected()
                && buf != null && buf.length > 0) {
                channel.send(buf);
            }
        } catch (Exception e) {
            log.error("Sending raw data", e);
        }
    }
}
