package org.supermmx.ginger.document;

/**
 * Caret in the document.
 *
 * @author SuperMMX
 */
public class GingerCaret {
    /**
     * Max width
     */
    private int width;
    /**
     * Max height
     */
    private int height;

    /**
     * Current column, starts from 0.
     */
    private int col;
    /**
     * Current row, starts from 0.
     */
    private int row;

    public GingerCaret(int width, int height) {
        this.width = width;
        this.height = height;

        col = 0;
        row = 0;
    }

    public void setWidth(int width) {
        this.width = width;
    }

    public void setHeight(int height) {
        this.height = height;
    }

    public int getColumn() {
        return col;
    }

    public int getRow() {
        return row;
    }

    /**
     * Move the caret to the specific position.
     *
     * @param col new column
     * @param row new row
     */
    public void moveTo(int col, int row) {
        if (col < 0) {
            col = 0;
        } else if (col >= width) {
            col = width - 1;
        }

        if (row < 0) {
            row = 0;
        } else if (row >= height) {
            row = height - 1;
        }

        this.col = col;
        this.row = row;
    }

    public void moveToColumn(int col) {
        moveTo(col, row);
    }

    public void moveToRow(int row) {
        moveTo(col, row);
    }

    public void forwardColumn() {
        forwardColumn(1);
    }

    /**
     * Forward the caret, stop at the right margin
     *
     * @param n n places
     */
    public void forwardColumn(int n) {
        if (col + n < width) {
            col += n;
        } else {
            col = width - 1;
        }
    }

    public void backwardColumn() {
        backwardColumn(1);
    }

    /**
     * Backward the caret, stop at the left margin
     *
     * @param n n places
     */
    public void backwardColumn(int n) {
        if (col - n >= 0) {
            col -= n;
        } else {
            col = 0;
        }
    }

    public void nextRow() {
        nextRow(1);
    }

    /**
     * Next lines.
     *
     * @param n n lines
     */
    public void nextRow(int n) {
        if (row + n < height) {
            row += n;
        } else {
            row = height - 1;
        }
    }

    public void previousRow() {
        previousRow(1);
    }

    /**
     * Previous lines.
     *
     * @param n n lines
     */
    public void previousRow(int n) {
        if (row - n >= 0) {
            row -= n;
        } else {
            row = 0;
        }
    }
}
