package org.supermmx.ginger.document;

/**
 * A segment in the document.
 *
 * @author SuperMMX
 */
public class GingerSegment {
    /**
     * The start index in the line, inclusive.
     */
    private int start;
    /**
     * The end index in the line, exclusive.
     */
    private int end;

    /**
     * Associated attribute
     */
    private GingerAttribute attr;

    public GingerSegment(int start, int end) {
        this(start, end, null);
    }

    public GingerSegment(int start, int end, GingerAttribute attr) {
        this.start = start;
        this.end = end;

        if (attr == null) {
            this.attr = new GingerAttribute();
        } else {
            this.attr = attr.cloneAttribute();
        }
    }

    public int getStart() {
        return start;
    }

    public void setStart(int start) {
        this.start = start;
    }

    public int getEnd() {
        return end;
    }

    public void setEnd(int end) {
        this.end = end;
    }

    public int getLength() {
        return end - start;
    }

    public GingerAttribute getAttribute() {
        return attr;
    }

    public void setAttribute(GingerAttribute attr) {
        this.attr = attr.cloneAttribute();
    }

    public boolean equals(Object o) {
        if (!(o instanceof GingerSegment)) {
            return false;
        }
        GingerSegment seg = (GingerSegment) o;

        if (start != seg.start) {
            return false;
        }

        if (end != seg.end) {
            return false;
        }

        return attr.equals(seg.attr);
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();

        sb.append("[").append(start).append(", ").append(end).append("): ");
        sb.append(attr);

        return sb.toString();
    }
}
