package org.supermmx.ginger.view;

import org.supermmx.ginger.GingerSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory; 

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import static java.awt.event.KeyEvent.*;

/**
 * Handle key input for view.
 *
 * @author SuperMMX
 */
public class GingerViewKeyHandler extends KeyAdapter{
    public static final byte ESC = (byte) 27; // 0x1b

    private static Log log = LogFactory.getLog(GingerViewKeyHandler.class);

    public GingerViewKeyHandler() {
    }

    /**
     * Process direction keys, which can be held processed
     */
    @Override
    public void keyPressed(KeyEvent e) {
        if (log.isTraceEnabled()) {
            log.trace("Pressed: Modifiers: ["
                      + KeyEvent.getKeyModifiersText(e.getModifiers())
                      + "], Key Code: [" + e.getKeyCode()
                      + "], Key Char: [" + e.getKeyChar() + "]");
        }

        if (!(e.getComponent() instanceof GingerView)) {
            return ;
        }
        // handle left/right/up/down/pageup/pagedown
        GingerView view = (GingerView) e.getComponent();
        byte[] data = null;
        switch (e.getKeyCode()) {
        case VK_LEFT:
            data = new byte[] { ESC, (byte) '[', (byte) 'D' };
            break;
        case VK_RIGHT:
            data = new byte[] { ESC, (byte) '[', (byte) 'C' };
            break;
        case VK_UP:
            data = new byte[] { ESC, (byte) '[', (byte) 'A' };
            break;
        case VK_DOWN:
            data = new byte[] { ESC, (byte) '[', (byte) 'B' };
            break;
        case VK_PAGE_UP:
            data = new byte[] { ESC, (byte) '[', (byte) '5', (byte) '~' };
            break;
        case VK_PAGE_DOWN:
            data = new byte[] { ESC, (byte) '[', (byte) '6', (byte) '~' };
            break;
        }
        if (data != null) {
            view.getSession().sendRawData(data);
        }
    }

    /**
     * Process general function keys.
     */
    @Override
    public void keyReleased(KeyEvent e) {
        if (log.isTraceEnabled()) {
            log.trace("Released: Modifiers: ["
                      + KeyEvent.getKeyModifiersText(e.getModifiers())
                      + "], Key Code: [" + e.getKeyCode()
                      + "], Key Char: [" + e.getKeyChar() + "]");
        }
        if (e.getComponent() instanceof GingerView) {
            byte[] data = null;

            GingerView view = (GingerView) e.getComponent();
            view.getSession().sendRawData(data);

            int code = e.getKeyCode();
            if (e.isControlDown()) {
                /*
                 * see http://en.wikipedia.org/wiki/Control_character#How_control_characters_map_to_keyboards
                 * and http://en.wikipedia.org/wiki/C0_and_C1_control_codes for more details
                 */
                // FIXME: the code is not correct for @^_
                if (code >= '@' && code <= '_') {
                    data = new byte[] { (byte) (code & 0x3f) };
                }
            } else {
                switch (code) {
                case VK_HOME:
                    data = new byte[] { ESC, (byte) '[', (byte) '1', (byte) '~' };
                    break;
                case VK_END:
                    data = new byte[] { ESC, (byte) '[', (byte) '4', (byte) '~' };
                    break;
                }
            }
            if (data != null) {
                view.getSession().sendRawData(data);
            }
        }
    }

    /**
     * Process normal characters
     */
    @Override
    public void keyTyped(KeyEvent e) {
        if (e.getComponent() instanceof GingerView) {
            if (e.getKeyChar() != CHAR_UNDEFINED
                &&  !e.isAltDown()) {
                byte[] data = null;
                try {
                    data = Character.toString(e.getKeyChar()).getBytes("GBK");
                } catch (Exception ex) {
                    log.error("Get data error from character " + e.getKeyChar(), ex);
                }
                GingerView view = (GingerView) e.getComponent();
                GingerSession session = view.getSession();
                if (data != null) {
                    if (session.isConnected()) {
                        session.sendRawData(data);
                    } else {
                        // reconnect
                        session.connect();
                    }
                }
            }
        }
    }
}
