package org.supermmx.ginger.document;

import static org.supermmx.ginger.document.GingerAttribute.*;

import org.junit.Test;
import static org.junit.Assert.*;

import java.util.List;
import java.util.ArrayList;

/**
 * Test cases for GingerLine.
 *
 * @author SuperMMX
 */
public class GingerLineTest {
    private GingerDocument doc = new GingerDocument(10, 10);

    /**
     * setValue case: merge all three adjacent segments.
     */
    @Test
    public void testSetValueMergeThree() {
        GingerLine expectLine = new GingerLine(doc,
                                               new GingerSegment(0, doc.getWidth(),
                                                                 GingerAttribute.createAttribute("33")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 6, GingerAttribute.createAttribute("34")),
                           new GingerSegment(6, doc.getWidth(), GingerAttribute.createAttribute("33")));

        doc.getCaret().moveTo(5, 0);
        doc.setAttribute(GingerAttribute.createAttribute("33"));

        line.setValue((byte) ' ');

        assertEquals(expectLine, line);
    }


    /**
     * setValue case: merge with previous segment.
     * The length of current segment is 1.
     */
    @Test
    public void testSetValueMergePrev0() {
        // length = 1
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 6,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(6, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));
        GingerLine line = new GingerLine(doc,
                                         new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                                         new GingerSegment(5, 6, GingerAttribute.createAttribute("34")),
                                         new GingerSegment(6, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(5, 0);
        doc.setAttribute(GingerAttribute.createAttribute("33"));

        line.setValue((byte) ' ');

        assertEquals(expectLine, line);
    }

    /**
     * setValue case: merge with previous segment
     * The length of current segment is greater than 1.
     */
    @Test
    public void testSetValueMergePrev1() {
        // length > 1
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 6,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(6, 7,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(7, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 7, GingerAttribute.createAttribute("34")),
                           new GingerSegment(7, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(5, 0);
        doc.setAttribute(GingerAttribute.createAttribute("33"));

        line.setValue((byte) ' ');

        assertEquals(expectLine, line);
    }

    /**
     * setValue case: merge with next segment.
     * The length of current segment is 1.
     */
    @Test
    public void testSetValueMergeNext0() {
        // length = 1
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 5,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));
        GingerLine line = new GingerLine(doc,
                                         new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                                         new GingerSegment(5, 6, GingerAttribute.createAttribute("34")),
                                         new GingerSegment(6, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(5, 0);
        doc.setAttribute(GingerAttribute.createAttribute("35"));

        line.setValue((byte) ' ');

        assertEquals(expectLine, line);
    }

    /**
     * setValue case: merge with next segment
     * The length of current segment is greater than 1.
     */
    @Test
    public void testSetValueMergeNext1() {
        // length > 1
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 5,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 6,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(6, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 7, GingerAttribute.createAttribute("34")),
                           new GingerSegment(7, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("35"));

        line.setValue((byte) ' ');

        assertEquals(expectLine, line);
    }

    /**
     * No merging, current segment length is 1.
     */
    @Test
    public void testSetValueNoMerge0() {
        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 5, GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 6, GingerAttribute.createAttribute("34")),
                           new GingerSegment(6, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(5, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.setValue((byte) ' ');

        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 5,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(5, 6,
                                             GingerAttribute.createAttribute("36")),
                           new GingerSegment(6, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        assertEquals(expectLine, line);
    }

    /**
     * No merging, column is at the start
     */
    @Test
    public void testSetValueNoMerge1() {
        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(4, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.setValue((byte) ' ');

        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 5,
                                             GingerAttribute.createAttribute("36")),
                           new GingerSegment(5, 8,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        assertEquals(expectLine, line);
    }

    /**
     * No merging, column is at the end
     */
    @Test
    public void testSetValueNoMerge2() {
        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(7, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.setValue((byte) ' ');

        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 7,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(7, 8,
                                             GingerAttribute.createAttribute("36")),
                           new GingerSegment(8, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        assertEquals(expectLine, line);
    }

    /**
     * No merging, column is in the middle
     */
    @Test
    public void testSetValueNoMerge3() {
        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.setValue((byte) ' ');

        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 6,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(6, 7,
                                             GingerAttribute.createAttribute("36")),
                           new GingerSegment(7, 8,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 0.
     *
     * Same attribute of the doc and current segment.
     */
    @Test
    public void testEraseToEnd0() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, doc.getWidth(),
                                             GingerAttribute.createAttribute("34")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("34"));

        line.erase(0);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 0.
     *
     * The column at the start of current segment
     */
    @Test
    public void testEraseToEnd1() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(4, 0);
        doc.setAttribute(GingerAttribute.createAttribute("35"));

        line.erase(0);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 0.
     *
     * The column is in the middle of current segment
     */
    @Test
    public void testEraseToEnd2() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 6,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(6, doc.getWidth(),
                                             GingerAttribute.createAttribute("36")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.erase(0);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 0.
     *
     * Column is at the start of the current segment, and doc attribute is
     * the same as previous segment. Merge with previous one.
     */
    @Test
    public void testEraseToEnd3() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, doc.getWidth(),
                                             GingerAttribute.createAttribute("33")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(4, 0);
        doc.setAttribute(GingerAttribute.createAttribute("33"));

        line.erase(0);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 1.
     *
     * Same attribute of the doc and current segment.
     */
    @Test
    public void testEraseToStart0() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, doc.getWidth(), GingerAttribute.createAttribute("34")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, doc.getWidth(),
                                             GingerAttribute.createAttribute("34")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("34"));

        line.erase(1);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 1.
     *
     * The column at the end of current segment
     */
    @Test
    public void testEraseToStart1() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 4,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(4, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(3, 0);
        doc.setAttribute(GingerAttribute.createAttribute("34"));

        line.erase(1);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 1.
     *
     * The column is in the middle of current segment
     */
    @Test
    public void testEraseToStart2() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, 7,
                                             GingerAttribute.createAttribute("36")),
                           new GingerSegment(7, 8,
                                             GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.erase(1);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 1.
     *
     * Column is at the end of the current segment, and doc attribute is
     * the same as previous segment. Merge with previous one.
     */
    @Test
    public void testEraseToStart3() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, doc.getWidth(),
                                             GingerAttribute.createAttribute("35")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(7, 0);
        doc.setAttribute(GingerAttribute.createAttribute("35"));

        line.erase(1);
        
        assertEquals(expectLine, line);
    }

    /**
     * Erase In Line with value 2.
     */
    @Test
    public void testEraseWholeLine() {
        GingerLine expectLine =
            new GingerLine(doc,
                           new GingerSegment(0, doc.getWidth(),
                                             GingerAttribute.createAttribute("36")));

        GingerLine line =
            new GingerLine(doc,
                           new GingerSegment(0, 4, GingerAttribute.createAttribute("33")),
                           new GingerSegment(4, 8, GingerAttribute.createAttribute("34")),
                           new GingerSegment(8, doc.getWidth(), GingerAttribute.createAttribute("35")));

        doc.getCaret().moveTo(6, 0);
        doc.setAttribute(GingerAttribute.createAttribute("36"));

        line.erase(2);
        
        assertEquals(expectLine, line);
    }
}
